<?php

namespace App\Knockout;

use Illuminate\Support\Facades\Cache;

class Subforum {

    public $id;
    public $name;
    public $description;

    public $threads = [];
    public Thread $thread;

    public static function unwrap($subforum)
    {
        $s = new self();

        $s->id              = $subforum->id;
        $s->name            = iconv('UTF-8', 'ASCII//TRANSLIT', $subforum->name);
        $s->description     = $subforum->description ?? null;

        // grab threads if available
        $s->threads = array_map(function($thread) {
            return Thread::unwrap($thread);
        }, $subforum->threads ?? []);

        // grab the last updated thread if available
        if (isset($subforum->lastPost)) {
            $s->thread = Thread::unwrap($subforum->lastPost->thread);
        }

        return $s;
    }

    private static function requestAll(): Dataset
    {
        $data = (new AbstractData)->httpGet('/subforum');
        $json = json_decode($data);
        $records = array_map(function($subforum) {
            return self::unwrap($subforum);
        }, $json->list);
        return (new Dataset($records));
    }

    public static function updateAll(): Dataset
    {
        $data = self::requestAll();
        Cache::put('subforums', $data, 3600);
        return $data;
    }

    public static function all(): Dataset
    {
        return Cache::get('subforums', function() {
            return self::updateAll();
        });
    }

    private static function requestOne(int $subforumId, int $page = 1): Dataset
    {
        $data = (new AbstractData)->httpGet(sprintf('/subforum/%s/%s', $subforumId, $page));
        $json = json_decode($data);
        $record = self::unwrap($json);
        return (new Dataset($record))
            ->setTotalRecords($json->totalThreads)
            ->setCurrentPage($json->currentPage)
            ->setRecordsPerPage(40);
    }

    public static function updateOne(int $subforumId, int $page = 1): Dataset
    {
        $key = sprintf('subforum-%u-%u', $subforumId, $page);
        $data = self::requestOne($subforumId, $page);
        Cache::put($key, $data, 3600);
        return $data;
    }

    public static function one(int $subforumId, int $page = 1): Dataset
    {
        $key = sprintf('subforum-%u-%u', $subforumId, $page);
        return Cache::get($key, function() use($subforumId, $page) {
            return self::updateOne($subforumId, $page);
        });
    }

}
